__all__ = ['JobPresetAttribute']

from .inference_source import InferenceSource


class JobPresetAttribute(object):

    def __init__(self, parent_preset, job_attribute,
                 inference_source = None,
                 value = None):

        self._parent_preset = parent_preset
        self._job_attribute = job_attribute
        self._id = parent_preset.get_id() + '_' + job_attribute.get_attribute().get_key()

        if inference_source is not None:
            if inference_source not in job_attribute.get_inference_sources():
                raise ValueError("Inference source must be set to a valid inference source.")
        else:
            inference_source = job_attribute.get_default_inference_source()
        self._inference_source = inference_source

        if value is None:
            value = job_attribute.get_attribute().get_default_value()
        self._value = value

    def get_parent_preset(self):
        return self._parent_preset

    def get_job_attribute(self):
        return self._job_attribute

    def get_id(self):
        return self._id

    def get_key(self):
        return self.get_job_attribute().get_attribute().get_key()

    def get_inference_source(self):
        return self._inference_source

    def set_inference_source(self, inference_source):

        if inference_source not in self.get_job_attribute().get_inference_sources():
            raise ValueError("Inference source must be set to a valid inference source.")

        self._inference_source = inference_source

    def get_value(self):
        return self._value

    def set_value(self, value):
        self._value = value

    def eval(self, plugin, source_project = None):

        inference_source = self.get_inference_source()
        attribute = self._job_attribute.get_attribute()

        # application source
        if inference_source == InferenceSource.get_application_inference_source():
            # app and app version attributes must be either user defined or constants if also application defined
            app = self._parent_preset.get_job_preset_attribute_by_key('app').get_value()
            version = self._parent_preset.get_job_preset_attribute_by_key('app_version').get_value()

            if self.get_key() == 'app':
                return app

            if self.get_key() == 'app_version':
                return version

            application_attribute_source = plugin.get_application_attribute_source()
            return application_attribute_source.get_attribute_value(app, version, self.get_key())

        # project source
        elif inference_source == InferenceSource.get_project_inference_source():
            if source_project is None:
                raise ValueError("Must provide source project for attribute '" + attribute.get_display_name() + "'")

            return source_project.get_attribute(attribute.get_key())

        # constant source
        elif inference_source == InferenceSource.get_constant_inference_source():
            return attribute.get_default_value()

        # user defined source
        elif inference_source == InferenceSource.get_user_defined_inference_source():
            return self.get_value()

        else:
            raise ValueError("Unknown inference source: " + unicode(inference_source))
