__all__ = 'Logger'

import datetime
import logging

from .log_item import LogItem


class Logger:
    """
    A class which logs messages using both pythons internal logging implementation and to a logging queue in the logging
    coordinator.
    """

    def __init__(self, name, logging_coordinator):
        self._logger = logging.getLogger(name)
        self._name = name
        self._logging_coordinator = logging_coordinator

    def _log(self, msg, level):

        # fail fast if msg is not a string
        if type(msg) is not unicode:
            raise ValueError("Logging message must be a unicode type not a " + unicode(type(msg)) + " type.", msg)

        log_item = LogItem(self._name, level, datetime.datetime.now(), msg)
        self._logging_coordinator.queue(log_item)

    def debug(self, msg, *args, **kwargs):
        self._log(msg, "DEBUG")
        #self._logger.debug(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        self._log(msg, "INFO")
        #self._logger.info(msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        self._log(msg, "WARNING")
        #self._logger.warning(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        self._log(msg, "ERROR")
        #self._logger.error(msg, *args, **kwargs)

    def exception(self, msg, *args, **kwargs):
        self._log(msg, "ERROR")
        #self._logger.exception(msg, *args, exc_info=True, **kwargs)

    def critical(self, msg, *args, **kwargs):
        self._log(msg, "ERROR")
        #self._logger.critical(msg, *args, **kwargs)
