__all__ = 'PackedSceneBuilder'

from abc import abstractmethod


class PackedSceneBuilder:

    def __init__(self, plugin):
        self._plugin = plugin

    def get_plugin(self):
        return self._plugin

    @abstractmethod
    def _build_scene_internal(self, project_name, product, product_version, attributes):
        raise NotImplementedError

    def get_current_scene(self, project_name, product, product_version, attributes=None):
        """
        todo make it so you only pass the project attribute and all other values are passed from it. Requires project
          attribute instances
        """

        if attributes is None:
            attributes = {}

        packed_project = self._build_scene_internal(project_name, product, product_version, attributes)

        packed_project.set_name(project_name)

        if attributes:
            for key, value in attributes.items():

                # If the key is not already in the packed project's attribute list
                if key not in packed_project.get_attributes():
                    # Then set the attribute
                    packed_project.set_attribute(key, value)

        return packed_project

    @abstractmethod
    def get_project_type_id(self):
        """ Returns the project type id of the current scene. """
        raise NotImplementedError

    @abstractmethod
    def get_product_type(self):
        """ Returns the project type for the current scene. """
        raise NotImplementedError

    @abstractmethod
    def get_product_version(self):
        """ Returns the project version for the current scene. """
        raise NotImplementedError
