__all__ = 'ProjectInstanceBuilder'

from abc import abstractmethod
from gm_meta_plugin.project_definition_schema import ProjectDefinitionSchema
from gm_meta_plugin.attribute_types import AttributeType
from gm_meta_plugin.errors.application_attribute_not_found import ApplicationAttributeNotFound


class ProjectInstanceBuilder(ProjectDefinitionSchema):

    def __init__(self, plugin, root_project_attribute_instance, project_attribute_instances):
        ProjectDefinitionSchema.__init__(self, plugin, root_project_attribute_instance, project_attribute_instances)

    def update_states(self, project_name=None, application_attribute_source=None):

        if application_attribute_source is None:
            application_attribute_source = self.get_plugin().get_application_attribute_source()

        if project_name is None:
            project_name = u""

        # set project name
        project_attribute_instance = self.get_root_project_attribute()
        project_attribute_instance.set_value(project_name)

        project_attribute_instance = project_attribute_instance.transition()
        while True:
            attribute = project_attribute_instance.get_attribute()

            if attribute.get_type() == AttributeType.NULL:
                break

            # attempt to set the project attributes value to the application inferred value
            try:
                app_inferred_value = application_attribute_source.get_attribute_value(None,
                                                                                      None,
                                                                                      project_attribute_instance.get_attribute().get_key())
            except ApplicationAttributeNotFound as e:
                # if the application source does not recognise this attribute key then use the default value
                app_inferred_value = project_attribute_instance.get_attribute().get_default_value()

            project_attribute_instance.set_value(app_inferred_value)
            project_attribute_instance = project_attribute_instance.transition(force_transition=True)

    @abstractmethod
    def get_project_attributes(self, attributes=None, force_transition=False, is_remote=False):
        raise NotImplementedError
