__all__ = 'ValidatedWidget'

from PySide2 import QtWidgets
from PySide2 import QtCore


class ValidatedWidget(QtWidgets.QWidget):

    def __init__(self, widget, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        self._widget = widget
        self._is_valid = True

        """ Create Widgets """
        self._warning_message = QtWidgets.QLabel(u"")
        self._warning_message.hide()
        self._warning_message.setProperty('invalid', True)
        self._warning_message.setWordWrap(True)

        """ Create Layouts """
        self._layout = QtWidgets.QVBoxLayout(self)
        self._layout.setContentsMargins(0,0,0,0)
        self._layout.setAlignment(QtCore.Qt.AlignVCenter)
        self._layout.addWidget(self._widget)

        self._warning_layout = QtWidgets.QHBoxLayout()
        self._warning_layout.setContentsMargins(0,0,0,0)
        self._warning_layout.addWidget(self._warning_message)

        self._layout.addLayout(self._warning_layout)

    def get_widget(self):
        return self._widget

    def align_warning_message_center(self, bool):
        if bool:
            self._warning_message.setAlignment(QtCore.Qt.AlignCenter)
        else:
            self._warning_message.setAlignment(QtCore.Qt.AlignLeft)

    def is_valid(self):
        return self._is_valid

    def set_validity_flag(self, is_valid):
        self._is_valid = is_valid

        if is_valid:
            self._warning_message.hide()
        else:
            self._warning_message.show()

    def set_to_valid(self):
        self.set_validity_flag(True)

    def set_to_invalid(self):
        self.set_validity_flag(False)

    def toggle_is_valid(self):
        self.set_validity_flag(not self.is_valid())

    def get_warning_message(self):
        return self._warning_message.text()

    def set_warning_message(self, message):
        self._warning_message.setText(message)
